<?php

namespace Drupal\dam\Entity;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\user\UserInterface;
use Drupal\entity_reference_revisions\EntityNeedsSaveInterface;
use Drupal\entity_reference_revisions\EntityNeedsSaveTrait;
use Drupal\Core\Entity\EntityPublishedTrait;
use Drupal\dam\FTPAccountInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Session\AccountInterface;

/**
 * Defines the FTPAccount entity.
 *
 * @ingroup FTPAccount
 *
 * @ContentEntityType (
 *   id = "ftp_account",
 *   label = @Translation("FTP Account"),
 *   bundle_label = @Translation("FTP Account Type"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\dam\Controller\FTPAccountListBuilder",
 *     "access" = "Drupal\dam\FTPAccountAccessControlHandler",
 *     "form" = {
 *       "default" = "Drupal\dam\Form\FTPAccountForm",
 *       "add" = "Drupal\dam\Form\FTPAccountForm",
 *       "edit" = "Drupal\dam\Form\FTPAccountForm",
 *       "delete" = "Drupal\dam\Form\FTPAccountDeleteForm",
 *     },
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "route_provider" = {
 *       "html" = "Drupal\dam\FTPAccountHtmlRouteProvider",
 *     },
 *  },
 *  base_table = "ftp_account",
 *  data_table = "ftp_account_field_data",
 *  admin_permission = "administer dam",
 *  entity_keys = {
 *    "id" ="id",
 *    "bundle" = "type",
 *    "published" = "status"
 *  },
 *  bundle_entity_type = "ftp_account_type",
 *  field_ui_base_route = "entity.ftp_account_type.edit_form",
 *  links = {
 *    "canonical" = "/ftp/{ftp_account}",
 *    "add-page" = "/ftp/ftp_account/add",
 *    "add-form" = "/ftp/ftp_account/add/{ftp_account_type}",
 *    "edit-form" = "/ftp/{ftp_account}/edit",
 *    "delete-form" = "/ftp/{ftp_account}/delete",
 *    "collection" = "/dam/ftp_accounts/list"
 *  },
 * )
 */
class FTPAccount extends ContentEntityBase implements FTPAccountInterface {

  use EntityNeedsSaveTrait;
  use EntityPublishedTrait;
  use EntityChangedTrait;


  /**
  * {@inheritdoc}
  */
  public function getParentEntity() {
    if (!isset($this->get('parent_type')->value) || !isset($this->get('parent_id')->value)) {
      return NULL;
    }

    $parent = \Drupal::entityTypeManager()->getStorage($this->get('parent_type')->value)->load($this->get('parent_id')->value);
    return $parent;
  }

  /**
  * {@inheritdoc}
  */
  public function setParentEntity(ContentEntityInterface $parent, $parent_field_name) {
    $this->set('parent_type', $parent->getEntityTypeId() );
    $this->set('parent_id', $parent->id() );
    $this->set('parent_field_name', $parent_field_name);
    return $this;
  }

  /**
  * {@inheritdoc}
  */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(EntityStorageInterface $storage, $update = TRUE) {
    $this->setNeedsSave(FALSE);
    parent::postSave($storage, $update);
  }

  /**
   * {@inheritdoc}
   */
  public function getTitle() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setTitle($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('uid')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('uid')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('uid', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('uid', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getType() {
    return $this->bundle();
  }

  /**
  * {@inheritdoc}
  */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['langcode'] = BaseFieldDefinition::create('language')
      ->setLabel(t('Language code'))
      ->setDescription(t('The paragraphs entity language code.'))
      ->setRevisionable(TRUE);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of the paragraphs author.'))
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDefaultValueCallback('Drupal\paragraphs\Entity\Paragraph::getCurrentUserId')
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Published'))
      ->setRevisionable(TRUE)
      ->setTranslatable(TRUE)
      ->setDefaultValue(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'visible',
        'type' => 'boolean',
        'weight' => 10,
      ])
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'weight' => 10,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayConfigurable('form', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Authored on'))
      ->setDescription(t('The time that the Paragraph was created.'))
      ->setRevisionable(TRUE)
      ->setTranslatable(TRUE)
      ->setDisplayOptions('form', array(
        'region' => 'hidden',
        'weight' => 0,
      ))
      ->setDisplayConfigurable('form', TRUE);

    $fields['parent_id'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Parent ID'))
      ->setDescription(t('The ID of the parent entity of which this entity is referenced.'))
      ->setSetting('is_ascii', TRUE);

    $fields['parent_type'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Parent type'))
      ->setDescription(t('The entity parent type to which this entity is referenced.'))
      ->setSetting('is_ascii', TRUE)
      ->setSetting('max_length', EntityTypeInterface::ID_MAX_LENGTH);

    $fields['username'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Username'))
      ->setDescription(t('An alternative password from the user account..'))
      ->setSettings(array(
        'default_value' => '',
        'max_length' => 255,
        'text_processing' => 0,
      ))
      ->setDisplayOptions('view', array(
        'label' => 'above',
        'type' => 'string',
        'weight' => -3,
      ))
      ->setDisplayOptions('form', array(
        'type' => 'string_textfield',
        'weight' => -3,
      ))
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['password'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Password'))
      ->setDescription(t('An alternative password from the user account..'))
      ->setSettings(array(
        'default_value' => '',
        'max_length' => 255,
        'text_processing' => 0,
      ))
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['home'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Home Directory'))
      ->setDescription(t('The home directory for the user, relative to the dam root.'))
      ->setSettings(array(
        'default_value' => '',
        'max_length' => 255,
        'text_processing' => 0,
      ))
      ->setDisplayOptions('view', array(
        'label' => 'above',
        'type' => 'string',
        'weight' => -3,
      ))
      ->setDisplayOptions('form', array(
        'type' => 'string_textfield',
        'weight' => -3,
      ))
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

  /**
  * Load the ftp account for the provided user id.
  */
  public static function getUserAccount($uid) {
    $ids = \Drupal::entityQuery('ftp_account')
      ->condition('uid', $uid)
      ->execute();

    if ($ids) {
      return entity_load('ftp_account', array_shift($ids) );
    }
    else {
      // If there is no account return null
      return NULL;
    }
  }

  /**
  *
  */
  public static function manageUserAccount(ContentEntityInterface $user) {
    $user_roles = $user->getRoles();
    $roles_permissions = user_role_permissions($user_roles);
    $config = \Drupal::config('dam.ftp_settings');

    $query = \Drupal::entityQuery('ftp_account')->condition('uid', $user->id(), '=');
    $results = $query->execute();

    if ( $user->hasPermission('create ftp account')
      && file_exists($config->get('dam_root_folder'))
      && $config->get('dam_root_folder') !== '/') {
      $account = entity_load('user', $user->id());

      if ($ftp_account_entity = entity_load('ftp_account', array_shift($results))) {
        $ftp_account_entity->set('username', $account->getUsername());
        $ftp_account_entity->set('password', $account->get('ftp_password')->value);
        $ftp_account_entity->set('home', $config->get('dam_root_folder'));
      }
      else {
        $ftp_account_entity = FTPAccount::create([
          'type' => 'default',
          'uid' => ['target_id' => $user->id()],
          'username' => $user->getUsername(),
          'password' => $account->get('ftp_password')->value,
          'status' => $user->get('status')->getValue()[0]['value'],
          'home' => $config->get('dam_root_folder'),
        ]);
      }
      if ($ftp_account_entity->save()) {
        $connection = \Drupal::database();
        $connection->query("UPDATE {ftp_account} SET `password` = PASSWORD('" . $account->get('ftp_password')->value  . "') WHERE uid = " . $user->id() )->execute();
      }
    }
    else if (!$user->hasPermission('create ftp account') || !($user->status->value) ){
      // Delete the ftp account entity if there is one.

      if ( count($results) ) {
        entity_delete_multiple('ftp_account', $results);
      }
    }

    // rebuild the ftp account file
    $ids = \Drupal::entityQuery('ftp_account')->execute();
    $entities = entity_load_multiple('ftp_account', $ids);
    $password = '';
    if ($entities) {
      $command = "openssl passwd -1 ";
      foreach($entities as $entity) {
        $return_pass = [];
        if ($return_pass1 = shell_exec($command . $entity->get('uid')->first()->entity->ftp_password->value)) {
          $password .= $entity->get('uid')->first()->entity->name->value . ":" . $return_pass1 . "\n";
        }
      }
    }

    $passwordFile = $config->get('dam_vsftp_user_file');
    if(file_exists($passwordFile)) {
      unlink($passwordFile);
    }
    file_put_contents($passwordFile, $password);
  }


}
